% Copyright 2018. All Rights Reserved
% Code by Shuang Li
% For Paper, "Atomic Norm Minimization for Modal Analysis from Random and Compressed Samples"
% IEEE transactions on signal processing, 2018
% by Shuang Li, Dehui Yang, Gongguo Tang, and Michael B. Wakin
% This code is used to reproduce Figure 3.

%%
clc; clear; close all;
trial=300;             % number of trials
Freq=[0.1 0.15 0.5];   % true frequencies
K=size(Freq,2);        % sparsity
Ts=1;                  % sampling interval
Nsample=80;            % number of samples
Nsensor=10;            % number of sensors
A=diag([1 2 3]);       % amplitudes
MMp=2:2:20;            % number of observed measurements per sensor
T=0:Ts:(Nsample-1)*Ts; 
T=T';
Q=exp(1j*2*pi*T*Freq);

c=ones(K,size(MMp,2))-0.01; % We view recovery as a success if MAC >= c
MAC_Asyn=zeros(K,size(MMp,2));
countj_Asyn=zeros(1,size(MMp,2));
MAC_Syn=zeros(K,size(MMp,2));
countj_Syn=zeros(1,size(MMp,2));
for t=1:trial
% ==========Close mode shapes===================
    MShape=randn(Nsensor,K);
    MShape(:,2)=MShape(:,1);
    MShape(1,2)=MShape(1,2)+0.3;  % The first two mode shapes are very close
    if Nsensor > 1
        MShape = MShape*diag(1./norms(MShape));
    else 
        MShape = ones(1,K);
    end
    Data=Q*A*transpose(MShape);   % data matrix
    for pp=1:size(MMp,2)
        fprintf('Trial=%d,Mp=%d\n',t,MMp(pp))
        Mp=MMp(pp);  % number of measurements
 % =================Asynchronous==========================       
        zz=randperm(Nsensor*Nsample,Mp*Nsensor);
        MData=zeros(Nsample,Nsensor);
        MData(zz)=Data(zz);                   % missing entries 
        cvx_begin quiet
            variable u0;
            variable u(Nsample-1) complex;
            variable Tu(Nsample,Nsample) hermitian;
            variable X(Nsample,Nsensor) complex;
            variable W(Nsensor,Nsensor) Hermitian; 
            dual variable Y0 
            minimize(0.5/Nsample*trace(Tu)+0.5*trace(W))
            subject to 
                Tu == toeplitz([u0;conj(u)],[u0;conj(u)]');
                Y0:[Tu   X; ...
                    X'   W] == semidefinite(Nsample+Nsensor,Nsample+Nsensor);
            X(zz)==MData(zz);
        cvx_end
        Y=-2*Y0(1:Nsample,Nsample+1:Nsample+Nsensor);    % dual solution
        f=0:0.01:50;  % frequency grid
        a=exp(1j*2*pi*T*f); % atoms
        QQ=Y'*a;   % dual polynomial
        if Nsensor > 1
            NormQ=norms(QQ);
        else
            NormQ=abs(QQ); 
        end

        [pks,locs] = findpeaks(NormQ,'minpeakdistance',1,'MinPeakHeight',0.99);
        if size(locs,2) < K
            Freq_est(1:size(locs,2))=f(locs);  % estimated frequencies
            B(:,1:size(locs,2))=QQ(:,locs);    % estimated mode shapes
        else 
            Freq_est=f(locs(1:K));
            B=QQ(:,locs(1:K));          
        end
        MAC_Asyn(:,pp)=diag(abs(MShape'*B));   % compute MAC
        
% =================Synchronous==========================       
        zz=randperm(Nsample,Mp);
        MData=zeros(Nsample,Nsensor);
        MData(zz,:)=Data(zz,:);                   % missing entries 
        cvx_begin quiet
            variable u0;
            variable u(Nsample-1) complex;
            variable Tu(Nsample,Nsample) hermitian;
            variable X(Nsample,Nsensor) complex;
            variable W(Nsensor,Nsensor) Hermitian; 
            dual variable Y0 
            minimize(0.5/Nsample*trace(Tu)+0.5*trace(W))
            subject to 
                Tu == toeplitz([u0;conj(u)],[u0;conj(u)]');
                Y0:[Tu   X; ...
                    X'   W] == semidefinite(Nsample+Nsensor,Nsample+Nsensor);
            X(zz,:)==MData(zz,:);
        cvx_end
        Y=-2*Y0(1:Nsample,Nsample+1:Nsample+Nsensor);    % dual solution
        f=0:0.01:50;
        a=exp(1j*2*pi*T*f);
        QQ=Y'*a;
        if Nsensor > 1
            NormQ=norms(QQ);
        else
            NormQ=abs(QQ);
        end

        [pks,locs] = findpeaks(NormQ,'minpeakdistance',1,'MinPeakHeight',0.99);
        if size(locs,2) < K
            Freq_est(1:size(locs,2))=f(locs);
            B(:,1:size(locs,2))=QQ(:,locs);
        else 
            Freq_est=f(locs(1:K));      % estimated frequencies
            B=QQ(:,locs(1:K));          % estimated mode shapes
        end
        MAC_Syn(:,pp)=diag(abs(MShape'*B));
    end
    indx_Asyn=(sum(MAC_Asyn>=c)==3);
    countj_Asyn(indx_Asyn)=countj_Asyn(indx_Asyn)+1
    indx_Syn=(sum(MAC_Syn>=c)==3);
    countj_Syn(indx_Syn)=countj_Syn(indx_Syn)+1
end
pj_Asyn=countj_Asyn./trial;
pj_Syn=countj_Syn./trial;
%% ==================Show results==========================
figure
lw=6;
fs=29;
plot(MMp,pj_Asyn,'r-','linewidth',lw);hold on;
plot(MMp,pj_Syn,'b--','linewidth',lw);hold on;
xlabel('Average $\#$ of measurements per sensor','Interpreter','LaTex','FontSize',fs)    
ylabel('Probability of successful recovery','FontSize',fs)
axis tight;
h = legend('Asynchronous','Synchronous');
set(h,'FontSize',fs);
prepare_plots(19,19,2,8.75)
set(gca,'FontSize',fs);


figure
lw=6;
fs=29;
subplot(3,1,1)
plot(real(MShape(:,1)),'r-','Linewidth',lw)
title('Correlated Mode Shapes','FontSize',fs)
ylabel('Mode Shape 1','FontSize',fs)
legend('True')
set(gca,'FontSize',fs);
axis tight;

subplot(3,1,2)
plot(real(MShape(:,2)),'r-','Linewidth',lw);
ylabel('Mode Shape 2','FontSize',fs)
set(gca,'FontSize',fs);
axis tight;

subplot(3,1,3)
plot(real(MShape(:,3)),'r-','Linewidth',lw);hold on;
xlabel('Node','FontSize',fs)
ylabel('Mode Shape 3','FontSize',fs)
set(gca,'FontSize',fs);
axis tight;




