% Copyright 2018. All Rights Reserved
% Code by Shuang Li
% For Paper, "Atomic Norm Minimization for Modal Analysis from Random and Compressed Samples"
% IEEE transactions on signal processing, 2018
% by Shuang Li, Dehui Yang, Gongguo Tang, and Michael B. Wakin
% This code is used to reproduce Figure 4(c).

%%
clc;clear; close all;

trial=10;                 % number of trials
Nsample=100;              % number of samples
Min_Sep=2/Nsample;        % minimum seperation
FFreq=0.03:Min_Sep:0.99;
KK=2:2:40;                % number of frequencies
MMp=4:4:80;               % Number of measurements
Nsensor=10;               % Number of sensors
T=0:Nsample-1;
T=T';
c=ones(max(KK),1)-0.01;
countj=zeros(size(MMp,2),size(KK,2));
for t=1:trial
    for mm=1:size(MMp,2)
        Mp=MMp(mm);
        MAC=zeros(max(KK),size(KK,2));
        for kk=1:size(KK,2)
            K=KK(kk);
            fprintf('Trial=%d,Mp=%d,K=%d\n',t,Mp,K)

            ff=randperm(size(FFreq,2),K);
            Freq=sort(FFreq(ff));    % frequencies
            
            A=diag(rand(K,1));       % amplitudes
            Q=exp(1j*2*pi*T*Freq);
            MShape=randn(Nsensor,K); % randoms mode shapes
            if Nsensor > 1
                MShape = MShape*diag(1./norms(MShape));
            else
                MShape = ones(1,K);
            end
            Data=Q*A*transpose(MShape);            % data matrix
            Phi=zeros(Mp*Nsensor,Nsample*Nsensor); % measurement matrix
            for l=1:Nsensor
                Phi((l-1)*Mp+1:l*Mp,(l-1)*Nsample+1:l*Nsample)=randn(Mp,Nsample);
            end
            Vyy=Phi*vec(Data(:,1:Nsensor));        % compressed measurements
            
            % =========== atomic norm minimization ==============
            cvx_begin quiet
            variable u0;
            variable u(Nsample-1) complex;
            variable Tu(Nsample,Nsample) hermitian;
            variable X(Nsample,Nsensor) complex;
            variable W(Nsensor,Nsensor) Hermitian;
            dual variable Y0
            minimize(0.5/Nsample*trace(Tu)+0.5*trace(W))
            subject to
            Tu == toeplitz([u0;conj(u)],[u0;conj(u)]');
            Y0:[Tu   X; ...
                X'   W] == semidefinite(Nsample+Nsensor,Nsample+Nsensor);
            Vyy==Phi*vec(X);    
            cvx_end
            
            Y=-2*Y0(1:Nsample,Nsample+1:Nsample+Nsensor); % dual soluton
            f=[0:0.0001:1];
            a=exp(1j*2*pi*T*f);
            
            QQ=Y'*a;         % dual polynomial
            if Nsensor > 1
                NormQ=norms(QQ);
            else
                NormQ=abs(QQ);
            end
            
            [pks,locs] = findpeaks(NormQ,'minpeakdistance',1,'MinPeakHeight',0.99);
            if size(locs,2) < K
                Freq_est(1:size(locs,2))=f(locs);  % recovered frequencies
                B(:,1:size(locs,2))=QQ(:,locs);    % recovered mode shapes
            else
                Freq_est=f(locs(1:K));
                B=QQ(:,locs(1:K));
            end
            mac=diag(abs(MShape'*B));
            MAC(1:size(mac,1),kk)=mac;
            if sum(MAC(1:K,kk)>=c(1:K))==K;      % successful recovery
                countj(mm,kk)=countj(mm,kk)+1;
            end
        end
        countj
    end
end

pj=countj./trial;   % probability of successfull recovery


%% ==================== Show Results =======================
figure;
fs=29;
prepare_plots(18,18,1,7.75);
imagesc(KK,MMp,pj);axis xy
xlabel('Sparsity ($K$)','Interpreter','LaTex','FontSize',fs);
ylabel('Number of measurements per sensor','Interpreter','LaTex','FontSize',fs)
title('Probability of successful recovery (ModeShapes)','FontSize',fs);
set(gcf, 'Color', 'w');
colormap pink;colorbar;
set(gca,'FontSize',fs);
